﻿using System;
using System.Data;
using System.IO;
using System.Xml;
using System.Collections;
using System.Threading;
using System.Web;

using Microsoft.SharePoint;
using Microsoft.SharePoint.WebControls;
using Microsoft.SharePoint.Utilities;

using NPOI;
using NPOI.POIFS;
using NPOI.HSSF;
using NPOI.Util;
using NPOI.HSSF.UserModel;
using NPOI.SS.UserModel;

namespace ExportCurrentListViewToExcel.Layouts.ExportCurrentListViewToExcel
{
    public partial class ExportToExcel : LayoutsPageBase
    {
        private readonly string m_ExcelSPLibraryTitle = "ExportToExcel";
        private readonly string m_LongOperLeadingHTML = "正在生成Excel文件...";
        private readonly string m_LongOperTrailingHTML = "正在响应您的请求. 这需要花费您一些时间.";
        protected void Page_Load(object sender, EventArgs e)
        {
            
        }

        protected override void OnPreRenderComplete(EventArgs e)
        {
            base.OnPreRenderComplete(e);
            var listIdToString = (String)Request.QueryString["listid"];
            var viewIdToString = (String)Request.QueryString["viewid"];
            var sourceUrl = (String)Request.QueryString["source"];
            var webUrl = this.Web.Url;
            ALongActionWithProgressIndicator(listIdToString, viewIdToString, sourceUrl);
        }

        /// <summary>
        /// 用来进行页面执行时的相应，并返回一个可供下载excel路径
        /// </summary>
        /// <param name="p_ListId">当前页面 List 的 guid</param>
        /// <param name="p_ViewId">当前视图的 guid</param>
        /// <param name="p_SourceUrl">执行成功后返回的Url</param>
        private void ALongActionWithProgressIndicator(string p_ListId, string p_ViewId, string p_SourceUrl)
        {
            try
            {
                using (var longOperation = new SPLongOperation(this.Page))
                {
                    longOperation.LeadingHTML = m_LongOperLeadingHTML;
                    longOperation.TrailingHTML = m_LongOperTrailingHTML;
                    longOperation.Begin();
                    var successUrl = GetListViewItems(p_ListId, p_ViewId);
                    var downloadUrl = this.Web.Url + "/_layouts/download.aspx?sourceurl=" + SPEncode.UrlEncode(successUrl) + "&FldUrl=";
                    longOperation.EndScript("window.frameElement.commonModalDialogClose(1, '" + downloadUrl + "');");
                }
            }
            catch (ThreadAbortException)
            {
            }
            catch (Exception ex)
            {
                SPUtility.TransferToErrorPage(ex.ToString());
            }
        }

        /// <summary>
        /// 获取当前视图的Items数据
        /// </summary>
        /// <param name="p_ListId">当前列表的 guid</param>
        /// <param name="p_ViewId">当前视图的 guid</param>
        /// <returns></returns>
        private string GetListViewItems(string p_ListId, string p_ViewId)
        {
                // 用户保存字段静态名字和显示名称对应的hash表
                var fieldTitleLists = new Hashtable();
                // 用于保存视图中字段名称
                var fieldNameLists = new ArrayList();

                var oList = this.Web.Lists[new Guid(p_ListId)];
                var view = oList.Views[new Guid(p_ViewId)];
                var oQuery = new SPQuery();
                oQuery.Query = view.Query;
                var items = oList.GetItems(oQuery);
                foreach (string fieldName in view.ViewFields)
                {
                    fieldNameLists.Add(fieldName);
                    // 利用栏的内部名称读取栏，并将静态名作为key，显示名作为值添加到哈希表
                    SPField oField = oList.Fields.GetFieldByInternalName(fieldName);
                    fieldTitleLists.Add(fieldName, oField.Title);
                }
                var ItemsData = items.GetDataTable();
                return BuildExcel(fieldTitleLists, oList.Title, view.Title, fieldNameLists, ItemsData);
        }

        /// <summary>
        /// 生成 excel文件
        /// </summary>
        /// <param name="p_FieldLists">视图中栏的集合</param>
        /// <param name="p_ListTitle">列表的标题，同时为excel的文件名</param>
        /// <param name="p_ViewTitle">视图的标题，同时为sheet的名称</param>
        /// <param name="p_FieldTitleLists">视图中栏对应的显示名称</param>
        /// <param name="p_ItemsData">视图中Items数据</param>
        /// <returns></returns>
        private string BuildExcel(Hashtable p_FieldLists, string p_ListTitle, string p_ViewTitle, ArrayList p_FieldTitleLists, DataTable p_ItemsData)
        {
            HSSFWorkbook hssfworkbook = new HSSFWorkbook();
            CellStyle headerStyle = SetHeaderStyle(hssfworkbook);
            CellStyle bodyStyle = SetBodyStyle(hssfworkbook);
            Sheet sheet = hssfworkbook.CreateSheet(p_ViewTitle);
            Row rowHeader = sheet.CreateRow(0);
            var columnCount = p_ItemsData.Columns.Count;
            for (int i = 0; i < columnCount; ++i)
            {
                // 如果表中的字段数字和已经等于视图中的字段个数，就不需要在处理，直接跳出。
                if (p_ItemsData.Columns.Count == p_FieldTitleLists.Count)
                    break;
                // 判断是否是视图中的字段。
                if (p_FieldTitleLists.Contains(p_ItemsData.Columns[i].ColumnName))
                {
                    var fieldTitle = p_FieldLists[p_ItemsData.Columns[i].ColumnName].ToString();
                    Cell cell = rowHeader.CreateCell(i);
                    cell.SetCellValue(fieldTitle);
                    cell.CellStyle = headerStyle;
                    sheet.SetColumnWidth(i, SetCulomnWidth(fieldTitle) * 256);
                }
                else
                {
                    p_ItemsData.Columns.Remove(p_ItemsData.Columns[i].ColumnName);
                    // 索引重置，因为已经删除了该字段，所以目前该字段位置保存的是下一个字段的名称，所以要重置，再次读取该值。
                    i -= 1;
                }
            }

            for (int i = 0; i < p_ItemsData.Rows.Count; i++)
            {
                Row rowBody = sheet.CreateRow(i + 1);
                for (int j = 0; j < p_ItemsData.Columns.Count; j++)
                {
                    Cell cell = rowBody.CreateCell(j);
                    cell.CellStyle = bodyStyle;
                    cell.SetCellValue(p_ItemsData.Rows[i][j].ToString());
                }
            }
            return CreateExcelFileToSharePoint(hssfworkbook, p_ListTitle);
        }

        /// <summary>
        /// 将生成excel文件添加网站的一个 文档库中，并提供可供下载的路径
        /// </summary>
        /// <param name="p_ExcelWorkBokk">生成的excel文件</param>
        /// <param name="p_fileName">文件名称</param>
        /// <returns>sharepoint网站中的 excel路径</returns>
        private string CreateExcelFileToSharePoint(HSSFWorkbook p_ExcelWorkBokk, string p_fileName)
        {
            //SPFile file = new SPFile();
            MemoryStream stream = new MemoryStream();
            p_ExcelWorkBokk.Write(stream);
            var fileUrl = string.Format("{0}.xls",p_fileName);
            SPFile file;
            SPList exportLibrary;
            this.Web.AllowUnsafeUpdates = true;
            try{
                exportLibrary = this.Web.Lists[m_ExcelSPLibraryTitle];
            }
            catch{
                Guid libraryGuid = this.Web.Lists.Add(m_ExcelSPLibraryTitle, "", SPListTemplateType.DocumentLibrary);
                exportLibrary = this.Web.Lists[libraryGuid];
            }
            var rootFolderUrl = exportLibrary.RootFolder.Url;
            SPFolder folder = this.Web.GetFolder(rootFolderUrl);
            file = folder.Files.Add(fileUrl, stream, true);
            this.Web.AllowUnsafeUpdates = false;
            return file.Url;
        }

        /// <summary>
        /// 设置 excel 中head区域的 格式
        /// </summary>
        /// <param name="p_workbook">excel文件</param>
        /// <returns>表格样式</returns>
        private CellStyle SetHeaderStyle (HSSFWorkbook p_workbook)
        {
            CellStyle style = p_workbook.CreateCellStyle();
            Font font = p_workbook.CreateFont();
            style.Alignment = HorizontalAlignment.CENTER;
            font.Boldweight = (short)FontBoldWeight.BOLD;
            style.SetFont(font);
            style.BorderBottom = CellBorderType.THIN;
            style.BorderLeft = CellBorderType.THIN;
            style.BorderRight = CellBorderType.THIN;
            style.BorderTop = CellBorderType.THIN;
            style.FillBackgroundColor = NPOI.HSSF.Util.HSSFColor.RED.index; //没效果
            return style;
        }

        /// <summary>
        /// 设置excel中 body区域的 样式
        /// </summary>
        /// <param name="p_workbook">excel文件</param>
        /// <returns>表格样式</returns>
        private CellStyle SetBodyStyle(HSSFWorkbook p_workbook)
        {
            CellStyle style = p_workbook.CreateCellStyle();
            style.BorderBottom = CellBorderType.THIN;
            style.BorderLeft = CellBorderType.THIN;
            style.BorderRight = CellBorderType.THIN;
            style.BorderTop = CellBorderType.THIN;
            return style;
        }

        /// <summary>
        /// 设置excel中head区域每个cell的宽度
        /// </summary>
        /// <param name="p_fieldName">栏名</param>
        /// <returns>宽度</returns>
        private int SetCulomnWidth(string p_fieldName)
        {
            return (2 * p_fieldName.Length + 1) * 3;
        }

    }
}
